﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public enum DIRECTION { UP, DOWN, LEFT, RIGHT, COUNT };

public class Movement : MonoBehaviour
{
    public enum MOVE_STYLE { FREE, CONSTRAINED };
    public enum MOVE_TYPE { RBODY, TRANS };

    public MOVE_STYLE move_style;
    public MOVE_TYPE move_type;

    public float velocity;
    public float translation_speed;
    public float raycast_distance;

    public LayerMask wall_mask;

    private DIRECTION direction;

    private Rigidbody2D rbody;
    private CircleCollider2D col;

    private Vector2 starting_position;

    private void Awake()
    {
        rbody = GetComponent<Rigidbody2D>();
        col = GetComponent<CircleCollider2D>();
        starting_position = transform.position;
    }

    // assume variable 'dir' has already been normalized
    public void Move(Vector2 dir)
    {
        Vector2 movement_vector = Vector2.zero;

        if(move_style == MOVE_STYLE.FREE)
        {
            movement_vector = dir;
        }
        else if(move_style == MOVE_STYLE.CONSTRAINED)
        {
            if(dir.x > 0)
            {
                if (CheckValidDirection(DIRECTION.RIGHT, raycast_distance))
                    movement_vector.x = dir.x;
            }
            else if(dir.x < 0)
            {
                if (CheckValidDirection(DIRECTION.LEFT, raycast_distance))
                    movement_vector.x = dir.x;
            }
            if(dir.y > 0)
            {
                if (CheckValidDirection(DIRECTION.UP, raycast_distance))
                    movement_vector.y = dir.y;
            }
            else if(dir.y < 0)
            {
                if (CheckValidDirection(DIRECTION.DOWN, raycast_distance))
                    movement_vector.y = dir.y;
            }
        }

        if (move_type == MOVE_TYPE.RBODY)
        {
            rbody.velocity = movement_vector * velocity;
        }
        else if (move_type == MOVE_TYPE.TRANS)
        {
            float movement_distance = translation_speed + col.radius;
            RaycastHit2D hit = Physics2D.Raycast(transform.position, movement_vector, movement_distance, wall_mask);
            Debug.DrawLine(transform.position, (Vector2)transform.position + (movement_vector*movement_distance));
            if(!hit)
                transform.Translate(movement_vector*translation_speed);
        }
    }

    private void Update()
    {
        // snap to axis


    }

    public bool CheckValidDirection(DIRECTION dir, float distance)
    {
        Vector2 cast_dir = Vector2.zero;

        switch(dir)
        {
            case DIRECTION.UP:
                cast_dir = new Vector2(0, 1);
                break;
            case DIRECTION.DOWN:
                cast_dir = new Vector2(0, -1);
                break;
            case DIRECTION.LEFT:
                cast_dir = new Vector2(-1, 0);
                break;
            case DIRECTION.RIGHT:
                cast_dir = new Vector2(1, 0);
                break;
        }

        RaycastHit2D hit = Physics2D.Linecast(transform.position, (Vector2)transform.position + cast_dir * distance, wall_mask);
        // returns true if hit is null, false otherwise
        return !hit;
    }

    public void Respawn()
    {
        transform.position = starting_position; 
    }

}
